@@ -1,8 +1,12 @@
 import re
+import datetime
+import time
+import random
 import contextlib
 import copy
 import itertools
 import os
+import shutil
 import glob
 import pickle
 import hashlib
@@ -65,7 +69,7 @@ def StanModel(
     force_compile=False,
     **kwargs,
 ):
-    """"Utility to load/save cached compiled Stan models using PyStan.
+    """ "Utility to load/save cached compiled Stan models using PyStan.
 
     Parameters
     ----------
@@ -541,7 +545,7 @@ def check_divergences(samples, quiet=False, return_diagnostics=False):
     ----------
     samples : ArviZ InferenceData instance
         Contains samples to be checked. Must contain both `posterior`
-        and `samples_stats`.
+        and `sample_stats`.
     quiet : bool, default False
         If True, do not print diagnostic result to the screen.
     return_diagnostics : bool, default False
@@ -582,7 +586,7 @@ def check_treedepth(samples, max_treedepth=10, quiet=False, return_diagnostics=F
     ----------
     samples : ArviZ InferenceData instance
         Contains samples to be checked. Must contain both `posterior`
-        and `samples_stats`.
+        and `sample_stats`.
     max_treedepth: int, default 10
         Maximum tree depth used in the calculation.
     quiet : bool, default False
@@ -600,10 +604,10 @@ def check_treedepth(samples, max_treedepth=10, quiet=False, return_diagnostics=F
         Number of samplers wherein the tree depth was greater than
         `max_treedepth`.
     """
-    # ArviZ v. 0.11.2 updated treedepth to be tree_depth
-    if az.__version__ >= "0.11.2":
+    # ArviZ v. 0.11.2 updated treedepth to be tree_depth and v. 0.11.4 reverted
+    try:
         n_too_deep = (samples.sample_stats.tree_depth.values >= max_treedepth).sum()
-    else:
+    except:
         n_too_deep = (samples.sample_stats.treedepth.values >= max_treedepth).sum()
 
     n_total = samples.sample_stats.dims["chain"] * samples.sample_stats.dims["draw"]
@@ -637,7 +641,7 @@ def check_energy(
     ----------
     samples : ArviZ InferenceData instance
         Contains samples to be checked. Must contain both `posterior`
-        and `samples_stats`.
+        and `sample_stats`.
     e_bfmi_rule_of_thumb : float, default 0.3 (as per cmdstan)
         Rule of thumb value for E-BFMI. If below this value, there may
         be cause for concern.
@@ -703,7 +707,7 @@ def check_ess(
     ----------
     samples : ArviZ InferenceData instance
         Contains samples to be checked. Must contain both `posterior`
-        and `samples_stats`.
+        and `sample_stats`.
     parameters : list of str, or None (default)
         Names of parameters to use. If None, use all parameters. For
         multidimensional parameters, each entry must be given
@@ -833,7 +837,7 @@ def check_rhat(
     ----------
     samples : ArviZ InferenceData instance
         Contains samples to be checked. Must contain both `posterior`
-        and `samples_stats`.
+        and `sample_stats`.
     parameters : list of str, or None (default)
         Names of parameters to use. If None, use all parameters. For
         multidimensional parameters, each entry must be given
@@ -946,7 +950,7 @@ def check_all_diagnostics(
     ----------
     samples : ArviZ InferenceData instance
         Contains samples to be checked. Must contain both `posterior`
-        and `samples_stats`.
+        and `sample_stats`.
     parameters : list of str, or None (default)
         Names of parameters to use in checking Rhat and ESS. If None,
         use all parameters. For multidimensional parameters, each entry
@@ -1149,6 +1153,7 @@ def sbc(
     cores=1,
     N=400,
     n_prior_draws_for_sd=1000,
+    samples_dir="sbc_samples",
     remove_sample_files=True,
     progress_bar=False,
 ):
@@ -1211,6 +1216,13 @@ def sbc(
         Number of prior draws to compute the prior standard deviation
         for a parameter in the prior distribution. This standard
         deviation is used in the shrinkage calculation.
+    samples_dir : str, default "sbc_samples"
+        Path to directory to store .csv and .txt files generated by
+        CmdStan. If the Stan models are made using PyStan, this is
+        ignored; it is only active if using CmdStanPy. The directory
+        specified here will NOT be destroyed after a calculation,
+        though the files within it may be, depending on the
+        `remove_sample_files` kwarg.
     remove_sample_files : bool, default True
         If True, remove .csv and .txt files generated by CmdStan. If
         the Stan models are made using PyStan, this is ignored; it is
@@ -1286,6 +1298,10 @@ def sbc(
     if "parallel_chains" in sampling_kwargs:
         del sampling_kwargs["parallel_chains"]
 
+    # Shut off CmdStanPy's progress bar by default
+    if "show_progress" not in sampling_kwargs:
+        sampling_kwargs["show_progress"] = False
+
     # Take a prior sample to infer data types
     if "pystan" in str(type(prior_predictive_model)):
         with disable_logging():
@@ -1302,7 +1318,10 @@ def sbc(
     elif "cmdstanpy" in str(type(prior_predictive_model)):
         with disable_logging():
             prior_sample = prior_predictive_model.sample(
-                data=prior_predictive_model_data, fixed_param=True, iter_sampling=1
+                data=prior_predictive_model_data,
+                fixed_param=True,
+                iter_sampling=1,
+                show_progress=False,
             )
         prior_sample = az.from_cmdstanpy(
             prior=prior_sample, prior_predictive=measured_data
@@ -1343,6 +1362,7 @@ def sbc(
         var_names,
         measured_data,
         n_prior_draws_for_sd,
+        samples_dir,
         remove_sample_files,
     )
 
@@ -1363,6 +1383,7 @@ def sbc(
                 posterior_predictive_var_names,
                 log_likelihood_var_name,
                 prior_sd,
+                samples_dir,
                 remove_sample_files,
             )
 
@@ -1425,6 +1446,7 @@ def _perform_sbc(args):
         posterior_predictive_var_names,
         log_likelihood_var_name,
         prior_sd,
+        samples_dir,
         remove_sample_files,
     ) = args
 
@@ -1444,24 +1466,19 @@ def _perform_sbc(args):
         )
     elif "cmdstanpy" in str(type(prior_predictive_model)):
         with disable_logging():
+            output_dir = _get_output_dir(samples_dir, "prior_pred")
             prior_sample_cmdstanpy = prior_predictive_model.sample(
-                data=prior_predictive_model_data, fixed_param=True, iter_sampling=1
+                data=prior_predictive_model_data,
+                fixed_param=True,
+                iter_sampling=1,
+                show_progress=False,
+                output_dir=output_dir,
             )
         prior_sample = az.from_cmdstanpy(
             prior=prior_sample_cmdstanpy, prior_predictive=measured_data
         )
         if remove_sample_files:
-            for flist in [
-                prior_sample_cmdstanpy.runset._csv_files,
-                prior_sample_cmdstanpy.runset._stdout_files,
-                prior_sample_cmdstanpy.runset._stderr_files,
-                prior_sample_cmdstanpy.runset._diagnostic_files,
-            ]:
-                for fname in flist:
-                    try:
-                        os.remove(fname)
-                    except:
-                        pass
+            _remove_sampling_files(prior_sample_cmdstanpy.runset)
 
     # Extract data generated from the prior predictive calculation
     for data in measured_data:
@@ -1489,8 +1506,12 @@ def _perform_sbc(args):
 
     elif "cmdstanpy" in str(type(posterior_model)):
         with disable_logging():
+            output_dir = _get_output_dir(samples_dir, "model")
             posterior_samples_cmdstanpy = posterior_model.sample(
-                data=posterior_model_data, parallel_chains=1, **sampling_kwargs
+                data=posterior_model_data,
+                parallel_chains=1,
+                output_dir=output_dir,
+                **sampling_kwargs,
             )
         posterior_samples = az.from_cmdstanpy(
             posterior=posterior_samples_cmdstanpy,
@@ -1500,17 +1521,7 @@ def _perform_sbc(args):
 
         # Clean out samples to save disk space
         if remove_sample_files:
-            for flist in [
-                posterior_samples_cmdstanpy.runset._csv_files,
-                posterior_samples_cmdstanpy.runset._stdout_files,
-                posterior_samples_cmdstanpy.runset._stderr_files,
-                posterior_samples_cmdstanpy.runset._diagnostic_files,
-            ]:
-                for fname in flist:
-                    try:
-                        os.remove(fname)
-                    except:
-                        pass
+            _remove_sampling_files(posterior_samples_cmdstanpy.runset)
 
     # Check diagnostics
     warning_code, diagnostics = check_all_diagnostics(
@@ -1576,6 +1587,7 @@ def _get_prior_sds(
     var_names,
     measured_data,
     n_prior_draws_for_sd,
+    samples_dir,
     remove_sample_files,
 ):
     """Compute standard deviations of prior parameters."""
@@ -1593,27 +1605,20 @@ def _get_prior_sds(
         )
     elif "cmdstanpy" in str(type(prior_predictive_model)):
         with disable_logging():
+            output_dir = _get_output_dir(samples_dir, "prior_pred")
             prior_samples_cmdstanpy = prior_predictive_model.sample(
                 data=prior_predictive_model_data,
                 fixed_param=True,
                 iter_sampling=n_prior_draws_for_sd,
+                show_progress=False,
+                output_dir=output_dir,
             )
         prior_samples = az.from_cmdstanpy(
             prior=prior_samples_cmdstanpy, prior_predictive=measured_data
         )
 
         if remove_sample_files:
-            for flist in [
-                prior_samples_cmdstanpy.runset._csv_files,
-                prior_samples_cmdstanpy.runset._stdout_files,
-                prior_samples_cmdstanpy.runset._stderr_files,
-                prior_samples_cmdstanpy.runset._diagnostic_files,
-            ]:
-                for fname in flist:
-                    try:
-                        os.remove(fname)
-                    except:
-                        pass
+            _remove_sampling_files(prior_samples_cmdstanpy.runset)
 
     # Compute prior sd's
     names, vals = _xarray_to_ndarray(prior_samples.prior)
@@ -1854,6 +1859,71 @@ def _screen_param(param, omit):
     return True
 
 
+def _get_output_dir(samples_dir, model_name):
+    """Hand-specify directory for sampling results. This is necessary
+    so that CmdStanPy doesn't try to access files with the same name
+    while doing multiprocessing."""
+    # Set up time stamp + random nine-digit number
+    now_ns = time.time_ns()
+    dt = datetime.datetime.fromtimestamp(now_ns // 1000000000)
+    s = dt.strftime("%Y%m%d%H%M%S") + "." + str(int(now_ns % 1000000000)).zfill(9)
+    s += "-" + str(random.randint(0, 1000000000)).zfill(9)
+
+    subdir = model_name + "-" + s
+    return os.path.join(samples_dir, subdir)
+
+
+def _remove_sampling_files(runset):
+    """Remove files laying around from sampling run."""
+
+    #### Alternatively, we could delete files one-by-one, in case other
+    #### stuff is in the directory we're deleting from. Here, we
+    #### obliterate everything in the directory.
+
+    # files = []
+    # try:
+    #     files.append(runset._csv_files)
+    # except:
+    #     pass
+
+    # try:
+    #     files.append(runset._stdout_files)
+    # except:
+    #     pass
+
+    # try:
+    #     files.append(runset._stderr_files)
+    # except:
+    #     pass
+
+    # try:
+    #     files.append(runset._diagnostic_files)
+    # except:
+    #     pass
+
+    # try:
+    #     files.append(runset._profile_files)
+    # except:
+    #     pass
+
+    # for flist in files:
+    #     for fname in flist:
+    #         try:
+    #             os.remove(fname)
+    #         except:
+    #             pass
+    #
+    # try:
+    #     os.rmdir(runset._output_dir)
+    # except:
+    #     pass
+
+    try:
+        shutil.rmtree(runset._output_dir)
+    except:
+        pass
+
+
 @contextlib.contextmanager
 def disable_logging(level=logging.CRITICAL):
     """Context manager for disabling logging when doing MCMC sampling.
